/*********************************************************************
* Software License Agreement (BSD License)
*
*  Copyright (c) 2010, Rice University
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*   * Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*   * Redistributions in binary form must reproduce the above
*     copyright notice, this list of conditions and the following
*     disclaimer in the documentation and/or other materials provided
*     with the distribution.
*   * Neither the name of the Rice University nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
*  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
*  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
*  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
*  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
*  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
*  POSSIBILITY OF SUCH DAMAGE.
*********************************************************************/

/* Author: Mark Moll */

#include "ompl/extensions/vortex/VortexEnvironment.h"
#include "ompl/util/Console.h"
#include <Vx/VxCollisionGeometry.h>

void ompl::control::VortexEnvironment::VortexCollisionSubscriber::notifyIntersect(
    Vx::VxUniverse::eIntersectEventType type,
    Vx::VxIntersectResult *result, Vx::VxDynamicsResponseInput* response)
{
    if (result)
    {
        bool valid = env_->isValidCollision(*result);
        if (env_->verboseContacts_)
        {
            Vx::VxCollisionGeometry *geom1, *geom2;
            static msg::Interface msg;
            result->getCollisionGeometryPair(&geom1, &geom2);
            const char* name1 = geom1->getName();
            const char* name2 = geom2->getName();
            msg.debug((valid ? "Valid" : "Invalid") + std::string(" contact between ") +
                (name1 ? name1 : "<unknown>") + " and " + (name2 ? name2 : "<unknown>"));
        }
        if (!valid) env_->invalidCollisionOccured_ = true;
    }
}

void ompl::control::VortexEnvironment::addCollisionSubscriber(
    VortexCollisionSubscriber* subscriber,
    Vx::VxUniverse::ResponseType type1, Vx::VxUniverse::ResponseType type2,
    Vx::VxUniverse::eIntersectEventType eventID)
{
    for (int i=world_->getUniverseCount()-1; i>=0; i--)
        world_->getUniverse(i)->addIntersectSubscriber(type1, type2, eventID, subscriber, 0);
}


void ompl::control::VortexEnvironment::addDefaultCollisionSubscriber()
{
    // if there already is a default collision subscriber, remove it first.
    if (collisionSubscriber_)
    {
        for (int i=world_->getUniverseCount()-1; i>=0; i--)
            world_->getUniverse(i)->removeIntersectSubscriber(
                Vx::VxUniverse::kResponsePart, Vx::VxUniverse::kResponsePart, Vx::VxUniverse::kEventFirst,
                collisionSubscriber_);
        delete collisionSubscriber_;
    }
    collisionSubscriber_ = new VortexCollisionSubscriber(this);
    addCollisionSubscriber(collisionSubscriber_);
}
